import { Component, ElementRef, ViewChild, OnInit, Input, OnDestroy, HostListener } from '@angular/core';

import {
  jelly,
  Label,
  MySprite, removeItemFromArr, ShapeRect, ShapeRectNew, tweenChange,

} from './Unit';
import { res, resAudio } from './resources';

import { Subject } from 'rxjs';
import { debounceTime } from 'rxjs/operators';

import TWEEN from '@tweenjs/tween.js';
import { Title } from './Title';
import { Listening } from './Listening';
import { Letter } from './Letter';
import { Exercises } from './Exercises.spreat';
import { getDefaultExercises } from '../bean/Exercises';
import { getDefaultTile } from '../bean/TitleBean';




@Component({
  selector: 'app-play',
  templateUrl: './play.component.html',
  styleUrls: ['./play.component.css']
})
export class PlayComponent implements OnInit, OnDestroy {

  @ViewChild('canvas', { static: true }) canvas: ElementRef;
  @ViewChild('wrap', { static: true }) wrap: ElementRef;

  // 数据
  data;

  ctx;

  canvasWidth = 1280; // canvas实际宽度
  canvasHeight = 720; // canvas实际高度

  canvasBaseW = 1280; // canvas 资源预设宽度
  canvasBaseH = 720;  // canvas 资源预设高度

  mx; // 点击x坐标
  my; // 点击y坐标


  // 资源
  rawImages = new Map(res);
  rawAudios = new Map(resAudio);

  images = new Map();

  animationId: any;
  winResizeEventStream = new Subject();

  audioObj = {};

  renderArr;

  bigCardRenderArr;

  mapScale = 1;

  canvasLeft;
  canvasTop;

  saveKey = 'YM5-10';

  canTouch = true;

  /**标题*/
  title: Title;

  listening: Listening;

  curAudio;

  letterArr: Array<Letter>;

  exercisesArr: Array<Exercises> = [];

  mask: ShapeRect;

  curExer;

  @HostListener('window:resize', ['$event'])
  onResize(event) {
    this.winResizeEventStream.next();
  }


  ngOnInit() {

    this.data = {};

    // 获取数据
    const getData = (<any>window).courseware.getData;
    getData((data) => {

      if (data && typeof data == 'object') {
        this.data = data;
      }
      else{
        let defaultData = '{"title":{"part1":"C","part2":"Read and point"},"letterArr":[{"isFill":"0","val":"re-","audioUrl":"http://staging-teach.cdn.ireadabc.com/8b5ecca5d4509e1b8a7e81e69d8db615.mp3"},{"isFill":"0","val":"un-","audioUrl":"http://staging-teach.cdn.ireadabc.com/30621df106455523573e3871a5a5c598.mp3"},{"isFill":"0","val":"dis-","audioUrl":"http://staging-teach.cdn.ireadabc.com/b44fa51172b19b555fda30717c773027.mp3"}],"exercisesArr":[{"leftAnswer":{"letterArr":[{"isFill":"0","val":"heat"}],"audioUrl":"http://staging-teach.cdn.ireadabc.com/b62600fd1e72b54d1d1c3bc71b7361af.mp3","picUrl":"http://staging-teach.cdn.ireadabc.com/ce38e5043b8127f03a6829829148f76d.png"},"rightAnswer":{"letterArr":[{"isFill":"1","val":"re"},{"isFill":"0","val":"heat"}],"audioUrl":"http://staging-teach.cdn.ireadabc.com/b62600fd1e72b54d1d1c3bc71b7361af.mp3","picUrl":"http://staging-teach.cdn.ireadabc.com/9901fec296d715fcbee1e8bf19bb3228.png"},"card":{"picUrl":"http://staging-teach.cdn.ireadabc.com/a886b150e8792d433ced3672f4df7dfb.jpg","audioUrl":"http://staging-teach.cdn.ireadabc.com/f95fd3997cdd67abd51a1ab45efd855d.mp3"},"picUrl":"http://staging-teach.cdn.ireadabc.com/d58dcbde56d5868dab8021a7a7548c0d.png"},{"leftAnswer":{"letterArr":[{"isFill":"0","val":"heat"}],"picUrl":"http://staging-teach.cdn.ireadabc.com/ce38e5043b8127f03a6829829148f76d.png"},"rightAnswer":{"letterArr":[{"isFill":"1","val":"re"},{"isFill":"0","val":"heat"}],"picUrl":"http://staging-teach.cdn.ireadabc.com/9901fec296d715fcbee1e8bf19bb3228.png"},"card":{"picUrl":"http://staging-teach.cdn.ireadabc.com/10dab2d49188db958a6a6e6b3ab7b5f5.png","audioUrl":"http://staging-teach.cdn.ireadabc.com/a252c069d04f8e04f91b2a1394072f30.mp3"},"picUrl":"http://staging-teach.cdn.ireadabc.com/d58dcbde56d5868dab8021a7a7548c0d.png"},{"leftAnswer":{"letterArr":[{"isFill":"0","val":"heat"}],"picUrl":"http://staging-teach.cdn.ireadabc.com/ce38e5043b8127f03a6829829148f76d.png"},"rightAnswer":{"letterArr":[{"isFill":"1","val":"re"},{"isFill":"0","val":"heat"}],"picUrl":"http://staging-teach.cdn.ireadabc.com/9901fec296d715fcbee1e8bf19bb3228.png"},"card":{"picUrl":"http://staging-teach.cdn.ireadabc.com/9fb5a33ee489517de5f2a2cf2d926a32.jpg","audioUrl":"http://staging-teach.cdn.ireadabc.com/30621df106455523573e3871a5a5c598.mp3"},"picUrl":"http://staging-teach.cdn.ireadabc.com/d58dcbde56d5868dab8021a7a7548c0d.png"},{"leftAnswer":{"letterArr":[{"isFill":"0","val":"heat"}],"picUrl":"http://staging-teach.cdn.ireadabc.com/10dab2d49188db958a6a6e6b3ab7b5f5.png"},"rightAnswer":{"letterArr":[{"isFill":"1","val":"re"},{"isFill":"0","val":"heat"}],"picUrl":"http://staging-teach.cdn.ireadabc.com/cb6a2f948c3febbb7881e6b6a7417811.png"},"card":{"picUrl":"http://staging-teach.cdn.ireadabc.com/3883964c48ea76d6a08386dfe1bf1204.jpg","audioUrl":"http://staging-teach.cdn.ireadabc.com/30621df106455523573e3871a5a5c598.mp3"},"picUrl":"http://staging-teach.cdn.ireadabc.com/d58dcbde56d5868dab8021a7a7548c0d.png"}],"audioUrl":"http://staging-teach.cdn.ireadabc.com/919b2c150b41b3306869ebe78ecb70b3.mp3"}';
        this.data = JSON.parse(defaultData);
      }
      //console.log('data:', JSON.stringify(data));

      // 初始化 各事件监听
      this.initListener();

      // 若无数据 则为预览模式 需要填充一些默认数据用来显示
      this.initDefaultData();

      // 初始化 音频资源
      this.initAudio();
      // 初始化 图片资源
      this.initImg();
      // 开始预加载资源
      this.load();

    }, this.saveKey);

  }

  ngOnDestroy() {
    window['curCtx'] = null;
    window.cancelAnimationFrame(this.animationId);
    this.cleanAudio();
  }


  cleanAudio() {
    if (this.audioObj) {
      for (const key in this.audioObj) {
        this.audioObj[key].pause();
      }
    }
  }


  load() {

    // 预加载资源
    this.loadResources().then(() => {
      window["air"].hideAirClassLoading(this.saveKey, this.data);
      this.init();
      this.update();
    });
  }


  init() {

    this.initCtx();
    this.initData();
    this.initView();
  }

  initCtx() {
    this.canvasWidth = this.wrap.nativeElement.clientWidth;
    this.canvasHeight = this.wrap.nativeElement.clientHeight;
    this.canvas.nativeElement.width = this.wrap.nativeElement.clientWidth;
    this.canvas.nativeElement.height = this.wrap.nativeElement.clientHeight;


    this.ctx = this.canvas.nativeElement.getContext('2d');
    this.canvas.nativeElement.width = this.canvasWidth;
    this.canvas.nativeElement.height = this.canvasHeight;

    window['curCtx'] = this.ctx;
    window['curImages'] = this.images;
  }






  updateItem(item) {
    if (item) {
      item.update();
    }
  }

  updateArr(arr) {
    if (!arr) {
      return;
    }
    for (let i = 0; i < arr.length; i++) {
      arr[i].update(this);
    }
  }







  initListener() {

    this.winResizeEventStream
      .pipe(debounceTime(500))
      .subscribe(data => {
        this.renderAfterResize();
      });


    // ---------------------------------------------
    const setParentOffset = () => {
      const rect = this.canvas.nativeElement.getBoundingClientRect();
      this.canvasLeft = rect.left;
      this.canvasTop = rect.top;
    };
    const setMxMyByTouch = (event) => {
      if (event.touches.length <= 0) {
        return;
      }
      if (this.canvasLeft == null) {
        setParentOffset();
      }
      this.mx = event.touches[0].pageX - this.canvasLeft;
      this.my = event.touches[0].pageY - this.canvasTop;
    };

    const setMxMyByMouse = (event) => {
      this.mx = event.offsetX;
      this.my = event.offsetY;
    };
    // ---------------------------------------------


    let firstTouch = true;

    const touchDownFunc = (e) => {
      if (firstTouch) {
        firstTouch = false;
        removeMouseListener();
      }
      setMxMyByTouch(e);
      this.mapDown(e);
    };
    const touchMoveFunc = (e) => {
      setMxMyByTouch(e);
      this.mapMove(e);
    };
    const touchUpFunc = (e) => {
      setMxMyByTouch(e);
      this.mapUp(e);
    };

    const mouseDownFunc = (e) => {
      if (firstTouch) {
        firstTouch = false;
        removeTouchListener();
      }
      setMxMyByMouse(e);
      this.mapDown(e);
    };
    const mouseMoveFunc = (e) => {
      setMxMyByMouse(e);
      this.mapMove(e);
    };
    const mouseUpFunc = (e) => {
      setMxMyByMouse(e);
      this.mapUp(e);
    };


    const element = this.canvas.nativeElement;

    const addTouchListener = () => {
      element.addEventListener('touchstart', touchDownFunc);
      element.addEventListener('touchmove', touchMoveFunc);
      element.addEventListener('touchend', touchUpFunc);
      element.addEventListener('touchcancel', touchUpFunc);
    };
    const removeTouchListener = () => {
      element.removeEventListener('touchstart', touchDownFunc);
      element.removeEventListener('touchmove', touchMoveFunc);
      element.removeEventListener('touchend', touchUpFunc);
      element.removeEventListener('touchcancel', touchUpFunc);
    };

    const addMouseListener = () => {
      element.addEventListener('mousedown', mouseDownFunc);
      element.addEventListener('mousemove', mouseMoveFunc);
      element.addEventListener('mouseup', mouseUpFunc);
    };
    const removeMouseListener = () => {
      element.removeEventListener('mousedown', mouseDownFunc);
      element.removeEventListener('mousemove', mouseMoveFunc);
      element.removeEventListener('mouseup', mouseUpFunc);
    };

    addMouseListener();
    addTouchListener();
  }


  playAudio(key, now = false, callback = null) {

    const audio = this.audioObj[key];
    if (audio) {
      if (now) {
        audio.pause();
        audio.currentTime = 0;
      }

      if (callback) {
        audio.onended = () => {
          callback();
        };
      }
      audio.play();
    }

    return audio;
  }



  loadResources() {
    const pr = [];
    this.rawImages.forEach((value, key) => {// 预加载图片

      const p = this.preload(value)
        .then(img => {
          this.images.set(key, img);
        })
        .catch(err => console.log(err));

      pr.push(p);
    });

    this.rawAudios.forEach((value, key) => {// 预加载音频

      const a = this.preloadAudio(value)
        .then(() => {
          // this.images.set(key, img);
        })
        .catch(err => console.log(err));

      pr.push(a);
    });
    return Promise.all(pr);
  }

  preload(url) {
    return new Promise((resolve, reject) => {
      const img = new Image();
      // img.crossOrigin = "anonymous";
      img.onload = () => resolve(img);
      img.onerror = reject;
      img.src = url;
    });
  }

  preloadAudio(url) {
    return new Promise((resolve, reject) => {
      const audio = new Audio();
      audio.oncanplay = (a) => {
        resolve(a);
      };
      audio.onerror = () => {
        reject();
      };
      audio.src = url;
      audio.load();
    });
  }


  renderAfterResize() {
    this.canvasWidth = this.wrap.nativeElement.clientWidth;
    this.canvasHeight = this.wrap.nativeElement.clientHeight;
    this.init();
  }





  checkClickTarget(target) {

    const rect = target.getBoundingBox();

    if (this.checkPointInRect(this.mx, this.my, rect)) {
      return true;
    }
    return false;
  }

  getWorlRect(target) {

    let rect = target.getBoundingBox();

    if (target.parent) {

      const pRect = this.getWorlRect(target.parent);
      rect.x += pRect.x;
      rect.y += pRect.y;
    }
    return rect;
  }

  checkPointInRect(x, y, rect) {
    if (x >= rect.x && x <= rect.x + rect.width) {
      if (y >= rect.y && y <= rect.y + rect.height) {
        return true;
      }
    }
    return false;
  }





  addUrlToAudioObj(key, url = null, vlomue = 1, loop = false, callback = null) {

    const audioObj = this.audioObj;

    if (url == null) {
      url = key;
    }

    this.rawAudios.set(key, url);

    const audio = new Audio();
    audio.src = url;
    audio.load();
    audio.loop = loop;
    audio.volume = vlomue;

    audioObj[key] = audio;

    if (callback) {
      audio.onended = () => {
        callback();
      };
    }
  }

  addUrlToImages(url) {
    this.rawImages.set(url, url);
  }






  // ======================================================编写区域==========================================================================





  /**
   * 添加默认数据 便于无数据时的展示
   */
  initDefaultData() {
    if (!this.data.title) {
      this.data.title = getDefaultTile();
    }

    if (!this.data.letterArr) {
      this.data.letterArr = [];
    }

    if (!this.data.exercisesArr) {
      this.data.exercisesArr = [];
    }

    for (let i = this.data.exercisesArr.length; i < 4; ++i) {

      let exercises = getDefaultExercises();

      this.data.exercisesArr.push(exercises);
    }

  }


  /**
   * 添加预加载图片
   */
  initImg() {

    for (let i = 0; i < this.data.exercisesArr.length; ++i) {

      let exercises = this.data.exercisesArr[i];

      if (exercises.picUrl) {
        this.addUrlToImages(exercises.picUrl);
      }

      if (exercises.leftAnswer && exercises.leftAnswer.picUrl) {
        this.addUrlToImages(exercises.leftAnswer.picUrl);
      }

      if (exercises.rightAnswer && exercises.rightAnswer.picUrl) {
        this.addUrlToImages(exercises.rightAnswer.picUrl);
      }

      if (exercises.card && exercises.card.picUrl) {
        this.addUrlToImages(exercises.card.picUrl);
      }
    }
  }

  /**
   * 添加预加载音频
   */
  initAudio() {



    // 音效
    this.addUrlToAudioObj('click', this.rawAudios.get('click'), 0.3);

    this.addUrlToAudioObj('open', this.rawAudios.get('open'));

    this.addUrlToAudioObj('turn', this.rawAudios.get('turn'));

    //标题发音
    if (this.data.title.audioUrl) {
      this.addUrlToAudioObj('titleAudio', this.data.title.audioUrl);
    }


    //听力材料
    if (this.data.audioUrl) {
      this.addUrlToAudioObj('listenAudio', this.data.audioUrl);
    }

    for (let i = 0; i < this.data.letterArr.length; ++i) {
      let letter = this.data.letterArr[i];

      if (letter.audioUrl) {
        this.addUrlToAudioObj(letter.audioUrl);
      }
    }


    //练习题音效
    for (let i = 0; i < this.data.exercisesArr.length; ++i) {

      let exercises = this.data.exercisesArr[i];

      if (exercises.leftAnswer && exercises.leftAnswer.audioUrl) {
        this.addUrlToAudioObj(exercises.leftAnswer.audioUrl);
      }

      if (exercises.rightAnswer && exercises.rightAnswer.audioUrl) {
        this.addUrlToAudioObj(exercises.rightAnswer.audioUrl);
      }

      if (exercises.card && exercises.card.audioUrl) {
        this.addUrlToAudioObj(exercises.card.audioUrl);
      }
    }

  }



  /**
   * 初始化数据
   */
  initData() {

    const sx = this.canvasWidth / this.canvasBaseW;
    const sy = this.canvasHeight / this.canvasBaseH;
    const s = Math.min(sx, sy);
    this.mapScale = s;


    this.renderArr = [];

    if (this.curAudio) {
      this.curAudio.pause();
    }

    this.curAudio = null;

    this.letterArr = [];

    this.bigCardRenderArr = [];

    this.exercisesArr = [];

  }



  /**
   * 初始化试图
   */
  initView() {


    this.initBg();

    this.initTitle();

    this.initBottomPart();

    this.initLetter();

    this.initExercises();
  }

  initBg(){
    const bg = new ShapeRect();
    bg.fillColor = "#f0f5f8";
    bg.setSize(this.canvasWidth, this.canvasHeight);
    this.renderArr.push(bg);
  }


  initBottomPart() {

    const listening = new Listening(this.audioObj['listenAudio'], this.images);
    listening.init(this.mapScale, this.mapScale);

    listening.bg.x = this.canvasWidth - listening.bg.width - 10 * this.mapScale;
    listening.bg.y = this.canvasHeight - listening.bg.height - 8 * this.mapScale;
    this.listening = listening;

    this.renderArr.push(listening.bg);
  }

  initTitle() {

    const title = new Title(this.data.title, this.images);

    title.init(this.mapScale, this.mapScale);
    title.audio = this.audioObj['titleAudio'];
    this.title = title;

    title.titleBg.y = 0;
    title.titleBg.x = 0;

    this.renderArr.push(title.titleBg);
  }


  initLetter() {

    let w = 680 * this.mapScale;

    let h = 67.6 * this.mapScale;

    let letterBg = new ShapeRect();
    letterBg.setSize(w, h);
    letterBg.alpha = 0;
    letterBg.y = 0;
    letterBg.x = this.canvasWidth / 2 - 184 * this.mapScale;
    this.renderArr.push(letterBg);

    let letterPic = new MySprite();
    letterPic.init(this.images.get('letter-bg'));
    letterPic.setScaleXY(this.mapScale);
    letterPic.x = w / 2;
    letterPic.y = h / 2;
    letterBg.addChild(letterPic);

    let letterArr = this.data.letterArr;

    let dW = 27 * this.mapScale;

    let tW = (letterArr.length - 1) * dW;


    let y = 13 * this.mapScale;

    for (let i = 0; i < letterArr.length; ++i) {

      let letter = new Letter();
      letter.init(this.images, letterArr[i], this.data.backgroudColor, this.mapScale, this.mapScale);
      letter.bg.y = y;
      letterBg.addChild(letter.bg);
      this.letterArr.push(letter);

      tW += letter.bg.width;
    }

    let startX = (letterBg.width - tW) / 2;

    for (let i = 0; i < this.letterArr.length; ++i) {
      let letter = this.letterArr[i];

      letter.bg.x = startX + i * dW;

      startX += letter.bg.width;
    }
  }

  initExercises() {

    let w = 500 * this.mapScale;

    let dW = 44 * this.mapScale;

    let dH = 38 * this.mapScale;

    let startX = (this.canvasWidth - (w * 2 + dW * (2 - 1))) / 2;

    let startY = this.canvasHeight / 2 - 247 * this.mapScale;

    let tempStartX = startX;


    const getView = (item, index, x = 0, y = 0) => {
      let exercises = new Exercises();

      exercises.index = index;

      exercises.init(this.images, this.data.backgroudColor, item, this.mapScale, this.mapScale);

      exercises.bg.x = x;

      exercises.bg.y = y;

      return exercises;
    }

    let index = 1;

    for (let i = 0; i < this.data.exercisesArr.length; ++i) {


      let exercises = getView(this.data.exercisesArr[i], index, startX, startY);

      if (index % 2 == 0) {

        //到下一行恢复起始x坐标
        startX = tempStartX;

        startY += exercises.bg.height + dH;

      }
      else {
        startX += exercises.bg.width + dW;
      }

      this.renderArr.push(exercises.bg);

      this.exercisesArr.push(exercises);

      index += 1;
    }



    const mask = new ShapeRect();
    mask.setSize(this.canvasWidth, this.canvasHeight);
    mask.visible = false;
    mask.alpha = 0.3
    mask.fillColor = "#000000";
    this.renderArr.push(mask);
    this.mask = mask;
  }



  mapDown(event) {

    if (!this.canTouch) {
      return;
    }

    if (this.curAudio) {
      if (this.curAudio != this.listening.audio) {
        this.curAudio.pause();
      }
    }

    if (this.checkClickTarget(this.listening.bg)) {

      this.curAudio = this.listening.play(() => {
        this.listening.reset();
      });

      return;
    }

    if (this.curAudio) {
      if (this.curAudio == this.listening.audio) {
        this.listening.pauseAudio();
      }
      else {
        this.curAudio.pause();
      }

    }

    if (this.checkClickTarget(this.title.titleBg)) {
      this.curAudio = this.title.playAudio();
      return;
    }

    for (let i = 0; i < this.letterArr.length; ++i) {
      let letter = this.letterArr[i];

      if (this.checkClickTarget(letter.bg)) {
        this.curAudio = this.playAudio(letter.data.audioUrl, true);
        break;
      }

    }


    for (let i = 0; i < this.exercisesArr.length; ++i) {

      let exer = this.exercisesArr[i];

      if (!exer.canClick) {
        continue;
      }

      if (this.checkClickTarget(exer.bg) && !exer.showed) {

        this.curExer = exer;

        this.playAudio('open', true);

        this.mask.visible = true;

        if (this.bigCardRenderArr.indexOf(exer.bigCard) == -1) {
          this.bigCardRenderArr.push(exer.bigCard);
        }

        exer.showBigCard(this.canvasWidth / 2, (this.canvasHeight - exer.bigCard.height) / 2, () => {

          //将其它词组设置为不可点击
          for (let j = 0; j < this.exercisesArr.length; ++j) {

            if (i != j) {

              this.exercisesArr[j].canClick = false;
            }
          }
        });

        break;

      }
    }

    if (this.curExer && this.curExer.canClickCard) {

      let target;

      let callback;

      if (this.checkClickTarget(this.curExer.overturnPic)) {
        target = this.curExer.overturnPic;
        callback = this.clickOverTurn.bind(this);
      }
      else if (this.checkClickTarget(this.curExer.closePic)) {
        target = this.curExer.closePic;
        callback = this.clickClose.bind(this);
      }
      else if (this.curExer.frontCard.visible && this.checkClickTarget(this.curExer.leftAnswer)) {
        target = this.curExer.leftAnswer;
        callback = this.clickLeft.bind(this);
      }
      else if (this.curExer.frontCard.visible && this.checkClickTarget(this.curExer.rightAnswer)) {
        target = this.curExer.rightAnswer;
        callback = this.clickRight.bind(this);
      }
      else if (this.curExer.backCard.visible && this.checkClickTarget(this.curExer.backCard)) {
        target = this.curExer.backCard;
        callback = this.clickBackCard.bind(this);
      }

      if (callback) {
        callback(target);
      }

      return;
    }
  }

  mapMove(event) {

  }

  mapUp(event) {

  }



  update() {

    // ----------------------------------------------------------
    this.animationId = window.requestAnimationFrame(this.update.bind(this));
    // 清除画布内容
    this.ctx.clearRect(0, 0, this.canvasWidth, this.canvasHeight);
    // tween 更新动画
    TWEEN.update();
    // ----------------------------------------------------------



    this.updateArr(this.renderArr);

    this.updateArr(this.bigCardRenderArr);

  }

  clickOverTurn(item: any) {
    if (!this.curExer) {
      return;
    }
    this.playAudio('turn', true);
    this.curExer.overturn(1);
  }

  clickClose(item: any) {
    if (!this.curExer) {
      return;
    }

    this.playAudio('turn', true);

    this.mask.visible = false;
    this.curExer.close(() => {
      //将词组设置为可点击
      for (let j = 0; j < this.exercisesArr.length; ++j) {
        this.exercisesArr[j].canClick = true;
      }

      removeItemFromArr(this.bigCardRenderArr, this.curExer.bigCard);

      this.curExer = null;
    });
  }

  clickLeft(item: any) {

    if (!this.curExer) {
      return;
    }

    this.canTouch = false;

    if (item.line) {

      if(item.line.alpha == 1){
        item.line.alpha = 0;
        item.text.visible = true;
        this.curExer.textAnimal(item.text, ()=>{
          this.canTouch = true;
        })
      }
      else{
        this.curExer.textAnimal(item.text, ()=>{
          this.canTouch = true;
        })
        this.curAudio = this.playAudio(this.curExer.data.leftAnswer.audioUrl, true);
      }
    }
    else{
      this.curAudio = this.playAudio(this.curExer.data.leftAnswer.audioUrl, true);
    }
  }

  clickRight(item: any) {
    if (!this.curExer) {
      return;
    }

    this.canTouch = false;

    if (item.line) {
      if(item.line.alpha == 1){
        item.line.alpha = 0;
        item.text.visible = true;
        this.curExer.textAnimal(item.text, ()=>{
          this.canTouch = true;
        })
       
      }
      else{
        this.curExer.textAnimal(item.text, ()=>{
          this.canTouch = true;
        })
        this.curAudio = this.playAudio(this.curExer.data.rightAnswer.audioUrl, true);
      }
    }
    else{
      this.curAudio = this.playAudio(this.curExer.data.rightAnswer.audioUrl, true);
    }
  }

  clickBackCard(item: any) {
    if (!this.curExer) {
      return;
    }

    this.curAudio = this.playAudio(this.curExer.data.card.audioUrl, true);
  }
}
